package nl.justobjects.toolkit.sys;

/**
 * Interval timer thread with callback to client.
 * <p>
 * <b>Purpose</b><br>
 * It calls the tick method in the TimerCallback interface after
 * a fixed number of milliseconds (indicated by the
 * interval variable). It measures the amount of time spent
 * in the tick method and adjusts for it.
 * <p>
 * <b>Implementation</b><br>
 * <p>
 * <b>Examples</b><br>
 * To start up a timer with this class, create it with
 * a callback and the number of milliseconds in the interval
 * then call the start method:
 * <PRE>
 *
 *	Timer timer = new Timer(this, 2000);	// 2 second interval
 *	timer.start();
 *
 * </PRE>
 * <p>
 * <br>
 * $Source: /var/cvs/justobjects/toolkit/src/nl/justobjects/toolkit/sys/Timer.java,v $<p>
 *
 * @version $Revision: 1.5 $
 * @author $Author: just $ - Just van den Broecke - Just Objects &copy;
 * @since $Date: 2003/01/06 00:23:49 $
 * @see TimerCallback
 */
public class Timer extends Object implements Runnable {
	static int num = 1;
	protected Thread timerThread;

	/** The number of milliseconds in the interval*/
	protected long interval;

	/** The callback interface containing the tick method */
	protected TimerCallback callback;

	public Timer() {
	}

	public Timer(TimerCallback callback) {
		this.callback = callback;
	}

	public Timer(long interval) {
		this.interval = interval;
	}

	public Timer(TimerCallback callback, long interval) {
		this.callback = callback;
		this.interval = interval;
	}

	/** returns the number of milliseconds in the interval */
	synchronized public long getInterval() {
		return interval;
	}

	/** sets the number of milliseconds in the interval
	 * @param newInterval the new number of milliseconds
	 */
	synchronized public void setInterval(long newInterval) {
		interval = newInterval;
	}

	/** returns the callback interface */
	public TimerCallback getCallback() {
		return callback;
	}

	/**
	 * changes the callback interface
	 * @param callback the new callback
	 */
	synchronized public void setCallback(TimerCallback callback) {
		this.callback = callback;
	}

	/// (re)starts the timer
	public void start() {
		timerThread = new Thread(this, "TimerThread" + num);
		num++;
		timerThread.start();
	}

	// stops timer
	public void stop() {
		timerThread.stop();
		timerThread = null;
	}

	public void run() {
		while (true) {

			// Check the current time
			long startTime = System.currentTimeMillis();

			// If there is a callback, call it
			if (callback != null) {
				callback.tick();
			}

			// Check the time again
			long endTime = System.currentTimeMillis();

			// The amount of time to sleep is the interval minus the time spent
			// in the tick routine
			long sleepTime = interval - (endTime - startTime);

			// If we've passed the next interval, hurry up and call the next tick
			if (sleepTime <= 0) continue;

			try {
				Thread.sleep(sleepTime);
			} catch (Exception insomnia) {
				// might as well ignore this exception
			}
		}
	}
}
