/****************************************************************
 * Copyright (C)2000 - Just van den Broecke - See license below *
 ****************************************************************/
package nl.justobjects.toolkit.xml;

import nl.justobjects.toolkit.io.IO;
import org.w3c.dom.*;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import java.io.File;
import java.io.PrintWriter;
import java.util.Vector;

/**
 * Utility methods for working with a DOM tree.
 * $Id: DOMUtil.java,v 1.16 2003/01/06 00:23:49 just Exp $
 * @author $Author: just $ - Just van den Broecke - Just Objects B.V. &copy;
 */
public class DOMUtil {
	/** Clears all childnodes in document */
	public static void clearDocument(Document document) {
		NodeList nodeList = document.getChildNodes();
		if (nodeList == null) {
			return;
		}
		int len = nodeList.getLength();
		for (int i = 0; i < len; i++) {
			document.removeChild(nodeList.item(i));
		}
	}

	/** Create empty document TO BE DEBUGGED!. */
	public static Document createDocument() {
		DocumentBuilder documentBuilder = null;
		//	System.setProperty("javax.xml.parsers.DocumentBuilderFactory", "org.apache.xerces.jaxp.DocumentBuilderFactoryImpl");
		try {
			documentBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
		} catch (ParserConfigurationException pce) {
			warn("ParserConfigurationException: " + pce);
			return null;
		}

		return documentBuilder.newDocument();
	}

	/** Copies all attributes from one element to another in the official way. */
	public static void copyAttributes(Element elementFrom, Element elementTo) {
		NamedNodeMap nodeList = elementFrom.getAttributes();
		if (nodeList == null) {
			// No attributes to copy: just return
			return;
		}

		Attr attrFrom = null;
		Attr attrTo = null;

		// Needed as factory to create attrs
		Document documentTo = elementTo.getOwnerDocument();
		int len = nodeList.getLength();

		// Copy each attr by making/setting a new one and
		// adding to the target element.
		for (int i = 0; i < len; i++) {
			attrFrom = (Attr) nodeList.item(i);

			// Create an set value
			attrTo = documentTo.createAttribute(attrFrom.getName());
			attrTo.setValue(attrFrom.getValue());

			// Set in target element
			elementTo.setAttributeNode(attrTo);
		}
	}


	/** Expands <tag id="xyz"> by inserting the elements from the content document */
	public static void expandIds(String tag, Element targetDocument, Document contentDocument) {
		Query query = new Query(targetDocument);
		query.addMatcherByTag(tag);
		Vector elementsToBeExpanded = query.query();

		p("expandIds: found " + elementsToBeExpanded.size() + " elements with tag=" + tag);
		if (elementsToBeExpanded.size() == 0) {
			warn("No elements with " + tag + " found");
			return;
		}

		Query contentQuery = null;

		int len = elementsToBeExpanded.size();
		for (int i = 0; i < len; i++) {
			Element elementPointer = (Element) elementsToBeExpanded.elementAt(i);
			String id = elementPointer.getAttribute("id");
			if ("".equals(id)) {
				warn("expandIds: no id attr for element=" + elementPointer.getTagName());
				return;
			}

			// Process single lesson to HTML slides
			p("expanding id=" + id);
			contentQuery = new Query(contentDocument);
			//contentQuery.addMatcherByTag(tag);
			contentQuery.addMatcherById(id);
			Vector elementContentList = contentQuery.query();
			p("found " + elementContentList.size() + " " + tag + " elements with id=" + id);

			if (elementContentList.size() == 0) {
				warn("Element " + tag + " with id=" + id + " not found");
				continue;
			}

			if (elementContentList.size() > 1) {
				warn("More than one element with " + tag + " with id=" + id + " found");
				continue;
			}
			Element newElement = (Element) elementContentList.elementAt(0);
			replaceElement(newElement, elementPointer);
		}
	}

	/** Expands <include> statements by inserting the files in the document */
	public static void expandIncludes(Document document) {
		expandIncludes(document, null, null);
	}

	/** Expands <include> statements by inserting the files in the document */
	public static void expandIncludes(Document document, String rootDir, String fileDir) {
		if ((fileDir == null) || (fileDir.equals(""))) {
			fileDir = System.getProperty("user.dir");
			//p("setting fileDir to user.dir: "+fileDir);
		}
		if ((rootDir == null) || (rootDir.equals(""))) {
			rootDir = fileDir;
			//p("setting fileDir to user.dir: "+fileDir);
		}


		// Get all <include/> elements
		Query query = new Query(document);
		query.addMatcherByTag("include");
		Vector elementList = query.query();
		p("expandIncludes: found " + elementList.size() + " elements");

		if (elementList.size() == 0) {
			p("expandIncludes: No included elements found");
			return;
		}

		// Replace each include element with the included file
		int len = elementList.size();
		for (int i = 0; i < len; i++) {
			Element element = (Element) elementList.elementAt(i);
			if (element == null) {
				//p("expandIncludes: element===null");
				return;
			}
			String href = element.getAttribute("href");
			String range = element.getAttribute("range");
			p("INCLUDE href=" + href + " range=" + range);

			String filePath = fileDir + File.separator + href;
			if (href.startsWith("/")) {
				// File is supposed to be relative to user.dir
				filePath = rootDir + File.separator + href;
			}

			// Show what we are doing
			// Only if a range specified determine the begin/end patterns to use.
			String startPattern = "", endPattern = "";
			if (range != null && !"".equals(range)) {
				// .java files use Java comment to hide pattern
				if (href.endsWith(".java") || href.endsWith(".cpp")) {
					startPattern = "//{" + range;
					endPattern = "//}" + range;

					// Markup files (.xml, .html, .xsl) use XML/HTML comment for hiding
				} else if (href.endsWith(".xml") || href.endsWith(".xsl") || href.endsWith(".dtd") ||
						href.endsWith(".html")) {
					startPattern = "<!--{" + range;
					endPattern = "<!--}" + range;

					// Batch files (msdos) (.BAT) use 'rem' comment for hiding
				} else if (href.endsWith(".BAT") || href.endsWith(".bat")) {
					startPattern = "rem {" + range;
					endPattern = "rem }" + range;

					// Range specified but no pattern for file extension !
				} else {
					p("ERROR: range specified, but file extension not supported: " + href);
				}

				// ASSERT: startPattern + endPattern for range completely specified
			}


			// Get the file as an array of strings (element per line)
			String[] lineArray = IO.FileToStringArray(filePath);
			String includeText = "";

			// Include the lines in the output line;
			// take range into account if specified.
			boolean inRange = false;
			for (int ix = 0; ix < lineArray.length; ix++) {

				// If range specified determine we are in it
				if (range != null && !"".equals(range)) {
					if (lineArray[ix].indexOf(startPattern) != -1) {
						// We are within a range
						inRange = true;

						// don't include range pattern itself
						continue;
					} else if (lineArray[ix].indexOf(endPattern) != -1) {
						// Range was ended
						inRange = false;
					}
				}
				String spaceBefore = "";
				// Only include line if within range or no range specified
				if (inRange || (range == null) || (range.equals(""))) {
					// Filter out any possible XML/HTML markup (or even < in .java!)
					// and include the line in the output line
					if (ix < 9) {
						spaceBefore = " ";
					}
					includeText = includeText + spaceBefore + (ix + 1) + ": " + lineArray[ix]; // already escaped by XML
				}
			}

			CDATASection includeTextNode = document.createCDATASection(includeText);

			// HACK: This stuff is needed to remove an empty text node
			// which created excess space on the first line !!!!
			Node previousSiblingNode = element.getPreviousSibling();
			if (previousSiblingNode != null && previousSiblingNode.getNodeType() == Node.TEXT_NODE) {
				element.getParentNode().removeChild(previousSiblingNode);
			}

			element.getParentNode().replaceChild(includeTextNode, element);
		}
	}

	public static Element getFirstElementByTagName(Document document, String tag) {
		// Get all elements matching the tagname
		NodeList nodeList = document.getElementsByTagName(tag);
		if (nodeList == null) {
			p("no list of elements with tag=" + tag);
			return null;
		}

		// Get the first if any.
		Element element = (Element) nodeList.item(0);
		if (element == null) {
			p("no element for tag=" + tag);
			return null;
		}
		return element;
	}

	public static Element getElementById(Document document, String id) {
		return getElementById(document.getDocumentElement(), id);
	}

	public static Element getElementById(Element element, String id) {
		return getElementById(element.getChildNodes(), id);
	}

	/** Get Element that has attribute id="xyz". */
	public static Element getElementById(NodeList nodeList, String id) {
		// Note we should really use the Query here !!

		Element element = null;
		int len = nodeList.getLength();
		for (int i = 0; i < len; i++) {
			Node node = (Node) nodeList.item(i);
			if (node.getNodeType() == Node.ELEMENT_NODE) {
				element = (Element) node;
				if (((Element) node).getAttribute("id").equals(id)) {
					// found it !
					break;
				}
			}
		}

		// returns found element or null
		return element;
	}

	/** Parses the specified URI and returns the document. */
	static public Document parse(String uri, boolean validate) throws Exception {
		DOMParser parser = DOMParser.getParser();
		if (validate) {
			parser.setProperty("validate", (validate ? "true" : "false"));
		}
		// Document document = null;
		return parser.parse(uri);
	}

	/** Parses the specified URI without DTD validation and returns the document. */
	static public Document parse(String uri) throws Exception {
		return parse(uri, false);
	}

	/** Prints an XHTML DOM as browser-compliant HTML */
	public static void printAsHTML(Document document, PrintWriter printWriter) {
		new TreeWalker(new HTMLPrintVisitor(printWriter)).traverse(document);
	}

	/** Prints an XML DOM.  */
	public static void printAsXML(Document document, PrintWriter printWriter) {
		new TreeWalker(new XMLPrintVisitor(printWriter)).traverse(document);
	}

	/** Replaces an element in document.  */
	public static void replaceElement(Element newElement, Element oldElement) {
		// Must be 1
		Node parent = oldElement.getParentNode();
		if (parent == null) {
			warn("replaceElement: no parent of oldElement found");
			return;
		}

		// Create a copy owned by the document
		ElementCopyVisitor ecv = new ElementCopyVisitor(oldElement.getOwnerDocument(), newElement);
		Element newElementCopy = ecv.getCopy();

		// Replace the old element with the new copy
		parent.replaceChild(newElementCopy, oldElement);
	}

	/** Write Document structure to XML file. */
	static public void document2File(Document document, String fileName) {
		new TreeWalker(new XMLPrintVisitor(fileName)).traverse(document);
	}

	public static void warn(String s) {
		p("DOMUtil: WARNING " + s);
	}

	public static void p(String s) {
		// System.out.println("DOMUtil: "+s);
	}

	private static void testCreateDocument() {
		Document document = createDocument();
	}

	private static void testReplaceElement() {
		Document targetDocument = null;
		Document contentDocument = null;
		try {
			targetDocument = parse("foo.xml");
			contentDocument = parse("bar.xml");
		} catch (Exception e) {
			p("Exception: " + e);
			e.printStackTrace();
			return;
		}
		Query query1 = new Query(targetDocument);
		query1.addMatcherById("bar2");
		Vector elementList1 = query1.query();

		Query query2 = new Query(contentDocument);
		query2.addMatcherById("bar2");
		Vector elementList2 = query2.query();

		Element oldElement = (Element) elementList1.elementAt(0);
		Element newElement = (Element) elementList2.elementAt(0);
		replaceElement(newElement, oldElement);
		// expandIds("module", targetDocument, contentDocument);
		document2File(targetDocument, "foo-exp.xml");
	}

	private static void testXMLFile(String fileName) {

		p("parsing " + fileName);
		try {
			Document document = parse(fileName);
		} catch (Exception e) {
			p("ERROR parsing file: " + e);
			return;
		}
		p("parse ok for " + fileName);
	}

	public static void main(String[] args) {
		// testReplaceElement();
		// testCreateDocument();
		testXMLFile(args[0]);
	}
}

/*
 * $Log: DOMUtil.java,v $
 * Revision 1.16  2003/01/06 00:23:49  just
 * moved devenv to linux
 *
 * Revision 1.15  2002/11/14 20:25:20  just
 * reformat of code only
 *
 * Revision 1.14  2002/06/18 10:30:02  just
 * no rel change
 *
 * Revision 1.13  2001/08/01 15:20:23  kstroke
 * fix for expand includes (added rootDir)
 *
 * Revision 1.12  2001/02/17 14:28:16  just
 * added comments and changed interface for expandIds()
 *
 * Revision 1.11  2000/12/09 14:35:35  just
 * added parse() method with optional DTD validation
 *
 * Revision 1.10  2000/09/21 22:37:20  just
 * removed print statements
 *
 * Revision 1.9  2000/08/28 00:07:46  just
 * changes for introduction of EntityResolverImpl
 *
 * Revision 1.8  2000/08/24 10:11:12  just
 * added XML file verfication
 *
 * Revision 1.7  2000/08/10 19:26:58  just
 * changes for comments only
 *
 *
 */

/*
   Copyright (C)2000 Just A. van den Broecke <just@justobjects.nl>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
