/****************************************************************
 * Copyright (C)2000 - Just van den Broecke - See license below *
 ****************************************************************/
package nl.justobjects.toolkit.xml;

import nl.justobjects.toolkit.io.IO;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;

import javax.xml.transform.Source;
import javax.xml.transform.URIResolver;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamSource;
import java.io.File;
import java.io.InputStream;

/**
 * Entity resolver implementation for SAX parsers.
 * $Id: EntityResolverImpl.java,v 1.13 2003/11/19 15:23:18 just Exp $
 * @author $Author: just $ - Just van den Broecke - Just Objects B.V. &copy;
 */
public class EntityResolverImpl implements EntityResolver, URIResolver {
	public static final String cpathPrefix = "cpath:/";
	public static final String cpathPrefix2 = "cpath://";

	public EntityResolverImpl() {
   // ClassLoader loader = EntityResolverImpl.class.getClassLoader();
    }

	/** Standard SAX callback. */
	public InputSource resolveEntity(String publicId, String systemId) {
		p("resolveEntity(): systemId="+systemId+" publicId="+publicId );
		String path = systemId;
		InputSource source = null;
		try {
			// Stupid windows expansion. Somehow this doesn't work even if
			// the file exists. Chop off the prefix and hope for the best...
			if (path.indexOf("file:///") != -1) {
				path = IO.replaceToken(path, "file:///", "");
			}

			if (new File(path).exists()) {
				p("resolveEntity[FILE]: systemId=" + systemId + " publicId=" + publicId);
				source = new InputSource(path);
			} else {
				// Hmm we cannot access it as a file, try several things...
				// Try forced resolving through the classpath
				p("resolveEntity[FILE]: systemId=" + systemId + " publicId=" + publicId + " trying CLASSPATH");
					InputStream is = getInputStreamFromClasspath(path, true);
				if (is != null) {
					p("resolveEntity[CLASSPATH]: systemId=" + systemId + " publicId=" + publicId);
					source = new InputSource(is);
				}
			}
		} catch (Exception e) {
			warn("resolveEntity Exception " + e);
		}

		if (source == null) {
			warn("resolveEntity; cannot resolve " + path);
			// System.exit(0);
		}

		return source;
	}

	/** Callback specifically for XSL (Xalan). */
	public Source resolve(String aHref, String aBase) {
		p("resolve(): aHref=" + aHref + " aBase=" + aBase);
		Source source = null;

		String path = aHref;
		try {
			// First try a regular file
			if (new File(path).exists()) {
				source = new StreamSource(path);
			} else {
				// Hmm we cannot access it as a file, try several things...
				// Try forced resolving through the classpath
				InputStream is = getInputStreamFromClasspath(path, true);
				if (is != null) {
					source = new DOMSource(DOMUtil.parse(path, false));
				}
			}
		} catch (Exception ignore) {
			warn("Exception " + ignore);
		}

		return source;
	}

	/** Find a resource through the CLASSPATH and create an InputStream when found. */
	private InputStream getInputStreamFromClasspath(String aResourcePath, boolean forced) {
		// Warn for "old-style" cpath:/ and cpath:// stuff
		String path = aResourcePath;
		if (path.startsWith("cpath:/")) {
			warn("[no-error] You are using old-style cpath:/[/] for " + path + " this is no longer required");
		}

		// Replace possible stupid Win separators
		path = path.replace('\\', '/');

		// First try a regular CLASSPATH search.
		InputStream is = EntityResolverImpl.class.getClassLoader().getResourceAsStream(path);

		// If not found and forced param set try more rigorous means...
		if (is == null && forced) {

			// Tricky: eat each trailing path prefix until we can (or cannot)
			// find the resulting path through the CLASSPATH.
			// This can resolve many resources specified as file:///, http:// cpath://
			// etc. It is not a clean solution but works for now until
			// we have a standardized XCatalog feature.
			while (path.indexOf("/") != -1) {
				if (path.charAt(0) == '/') {
					path = path.substring(1, path.length());
				}

				// Try through our own classloader
				// p("path=" + path);
				is = EntityResolverImpl.class.getClassLoader().getResourceAsStream(path);
				if (is != null) {
					// found it!
					break;
				}

				if (path.indexOf('/') != -1) {
					path = path.substring(path.indexOf("/"), path.length());
				}
			}
		}

		return is;
	}

	private static void warn(String s) {
		System.out.println("WARN[EntityResolverImpl]: " + s);
	}

	public static void p(String s) {
		//System.out.println("EntityResolverImpl-: " + s);
	}

	public static void main(String args[]) {
		try {
			DOMParser parser = DOMParser.getParser();
			parser.parse(args[0]);
		} catch (Exception e) {
			System.out.println("EXCEPTION: " + e);
		}
	}
}

/*
 * $Log: EntityResolverImpl.java,v $
 * Revision 1.13  2003/11/19 15:23:18  just
 * *** empty log message ***
 *
 * Revision 1.12  2003/11/19 15:15:15  just
 * many changes for using JAXP (with xerces)
 *
 * Revision 1.11  2003/01/06 00:23:49  just
 * moved devenv to linux
 *
 * Revision 1.10  2002/11/14 20:25:20  just
 * reformat of code only
 *
 * Revision 1.9  2002/09/30 22:03:00  just
 * fix for resolving file:/// system ids
 *
 * Revision 1.8  2002/02/04 23:27:50  just
 * mainly changes to find resources through CLASSPATH (no cpath:/ anymore)
 *
 * Revision 1.7  2002/01/03 15:33:16  just
 * changes required for upgrade to Xalan 2.2D14
 *
 * Revision 1.6  2001/12/13 23:11:16  just
 * modified classloading by using class.getClassLoader()
 *
 * Revision 1.5  2000/12/09 14:38:54  just
 * removed print statements
 *
 * Revision 1.4  2000/11/23 12:53:01  just
 * various changes for entity resolvement
 *
 * Revision 1.3  2000/09/21 22:37:20  just
 * removed print statements
 *
 * Revision 1.2  2000/09/14 11:38:27  just
 * updates
 *
 * Revision 1.1  2000/08/28 00:07:46  just
 * changes for introduction of EntityResolverImpl
 *
 *
 */

/*
   Copyright (C)2000 Just A. van den Broecke <just@justobjects.nl>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
