/****************************************************************
 * Copyright (C)2000 - Just van den Broecke - See license below *
 ****************************************************************/
package nl.justobjects.toolkit.xml;

import org.w3c.dom.*;

import java.util.Vector;

/**
 * Implements queries on DOM trees.
 * <p>
 * This class is implemented using the Visitor and Strategy patterns.
 * The Visitor pattern is used to traverse a DOM tree. The Strategy pattern
 * is used to configure a series of "matchers" (interface ElementMatcher).
 * As the DOM tree is visited each of the matchers is called. If
 * all matchers match the visited element is added to the result vector.
 * NB: currently there is no support for OR-ed matchers (to keep it simple).
 *
 * A query is done as follows
 * (1) create a Query object using the target Document/Element,
 * (2) add the matchers by calling query.addMatcherXYZ(args)
 * (3) perform the query using query.query()
 *
 * $Id: Query.java,v 1.5 2003/01/06 00:23:49 just Exp $
 *
 * @author $Author: just $ - Just van den Broecke - Just Objects B.V. &copy;
 */
public class Query extends DefaultVisitor {
	Vector elementMatchers = new Vector(5);
	Vector result = new Vector(5);
	Element targetElement = null;

	public Query(Element theTargetElement) {
		setTarget(theTargetElement);
	}

	public Query(Document theDocument) {
		setTarget(theDocument);
	}

	public void setTarget(Element theTargetElement) {
		targetElement = theTargetElement;
	}

	public void setTarget(Document theDocument) {
		targetElement = theDocument.getDocumentElement();
	}

	public void resetMatchers() {
		elementMatchers.removeAllElements();
		elementMatchers = new Vector(5);
	}

	public void resetResult() {
		result.removeAllElements();
		result = new Vector(5);
	}

	public void reset() {
		resetMatchers();
		resetResult();
	}

	/** Perform the actual query using the TreeWalker/Visitor. */
	public Vector query() {
		new TreeWalker(this).traverse(targetElement);
		return (Vector) result.clone();
	}

	/** Add matcher for attribute id="xyz". */
	public void addMatcherById(String id) {
		addMatcherByAttributeNameAndValue("id", id);
	}

	/** Add matcher for tag name. */
	public void addMatcherByTag(String tag) {
		addMatcher(new MatcherByTag(tag));
	}

	/** Add matcher for attribute name match. */
	public void addMatcherByAttributeName(String name) {
		addMatcher(new MatcherByAttributeName(name));
	}

	/** Add matcher for attribute N/V match. */
	public void addMatcherByAttributeNameAndValue(String name, String value) {
		addMatcher(new MatcherByAttributeNameAndValue(name, value));
	}

	/** Add matcher for attribute N/V match. */
	public void addMatcherByAttributeNameAndValueStartsWith(String name, String value) {
		addMatcher(new MatcherByAttributeNameAndValueStartsWith(name, value));
	}

	/** Add matcher providing an example Element. */
	public void addMatcherByExample(Element element) {
		addMatcher(new MatcherByExample(element));
	}

	/** Add matcher for Node type. */
	public void addMatcherByNodeType(short theNodeType) {
		addMatcher(new MatcherByNodeType(theNodeType));
	}

	/** Add matcher for text equal. */
	public void addMatcherByText(String text) {
		addMatcher(new MatcherByText(text));
	}

	/** Add matcher for text containing the string. */
	public void addMatcherByTextContains(String text) {
		addMatcher(new MatcherByTextContains(text));
	}

	/** Add custom matcher. */
	public void addMatcher(ElementMatcher elementMatcher) {
		elementMatchers.addElement(elementMatcher);
	}

	/** Callback from TreeWalker. */
	public void visitElementPost(Element element) {
		Element candidate = null;
		int count = 0;
		for (count = 0; count < elementMatchers.size(); count++) {
			if (!((ElementMatcher) elementMatchers.elementAt(count)).match(element)) {
				// At least one did not match --> no match
				break;
			} else {
				// Save in case we match all
				candidate = element;
			}
		}

		// We must have matched ALL matchers (AND condition)
		if ((count == elementMatchers.size()) && (candidate != null)) {
			addResult(candidate);
		}
	}

	/** Add element to final result. */
	protected void addResult(Element element) {
		// Make results unique
		if (!result.contains(element)) {
			result.addElement(element);
		}
	}

	//
	// Inner classes for each of the ElementMatchers.
	//

	private static class MatcherByTag implements ElementMatcher {
		String tag = "";

		public MatcherByTag(String theTag) {
			setTarget(theTag);
		}

		public MatcherByTag(Element targetElement) {
			this(targetElement.getTagName());
		}

		public void setTarget(String theTag) {
			tag = theTag;
		}

		public boolean match(Element element) {
			return element.getTagName().equalsIgnoreCase(tag);
		}
	}

	private static class MatcherByAttributeName implements ElementMatcher {
		String name = "";

		public MatcherByAttributeName() {
		}

		public MatcherByAttributeName(String theName) {
			setTarget(theName);
		}

		public void setTarget(String theName) {
			name = theName;
		}

		public boolean match(Element element) {
			return element.getAttributeNode(name) != null;
		}
	}

	private static class MatcherByAttributeNameAndValue extends MatcherByAttributeName {
		String value = "";

		public MatcherByAttributeNameAndValue() {
		}

		public MatcherByAttributeNameAndValue(String theName, String theValue) {
			super(theName);
			value = theValue.toLowerCase();
		}

		public void setTarget(String name, String theValue) {
			super.setTarget(name);
			value = theValue;
		}

		public boolean match(Element element) {
			return super.match(element) && element.getAttributeNode(name).getValue().toLowerCase().equals(value);
		}
	}

	private static class MatcherByAttributeNameAndValueStartsWith extends MatcherByAttributeNameAndValue {

		public MatcherByAttributeNameAndValueStartsWith(String theName, String theValue) {
			super(theName, theValue);
		}

		public boolean match(Element element) {
			Attr attr = element.getAttributeNode(name);
			return attr != null && attr.getValue().toLowerCase().startsWith(value);
		}
	}

	private static class MatcherByExample implements ElementMatcher {
		Element example;

		public MatcherByExample(Element theExample) {
			example = theExample;
		}

		public boolean match(Element element) {
			// 1. The tagnames must match
			if (!new MatcherByTag(example).match(element)) {
				return false;
			}

			// 2. Each of the attributes in the example must match name and value
			MatcherByAttributeNameAndValue attrMatcher = new MatcherByAttributeNameAndValue();
			NamedNodeMap attributeMap = example.getAttributes();
			for (int i = 0; i < attributeMap.getLength(); i++) {
				Attr attr = (Attr) attributeMap.item(i);
				attrMatcher.setTarget(attr.getName(), attr.getValue());
				if (!attrMatcher.match(element)) {
					return false;
				}
			}

			// Tagnames and attributes of the example match
			return true;
		}
	}

	private static class MatcherByNodeType implements ElementMatcher {
		short nodeType = Node.ELEMENT_NODE;

		public MatcherByNodeType() {
		}

		public MatcherByNodeType(short theNodeType) {
			setTarget(theNodeType);
		}

		public void setTarget(short theNodeType) {
			nodeType = theNodeType;
		}

		public boolean match(Element element) {
			return element.getNodeType() == nodeType;
		}
	}


	private static class MatcherByText implements ElementMatcher {
		String text;

		public MatcherByText(String theText) {
			setTarget(theText);
		}

		public void setTarget(String theText) {
			text = theText.trim();
		}

		public boolean match(Element element) {
			element.normalize();
			NodeList children = element.getChildNodes();
			for (int i = 0; i < children.getLength(); i++) {
				if (children.item(i).getNodeType() == Node.TEXT_NODE) {
					return matchText(((Text) children.item(i)).getData());
				}
			}
			return true;
		}

		protected boolean matchText(String elementText) {
			return elementText.trim().toLowerCase().equalsIgnoreCase(text);
		}
	}

	private static class MatcherByTextContains extends MatcherByText {

		public MatcherByTextContains(String theText) {
			super(theText);
		}

		protected boolean matchText(String elementText) {
			return elementText.toLowerCase().indexOf(text.toLowerCase()) != -1;
		}
	}

	static public void main(String[] args) {
		try {
			Query query = new Query(DOMUtil.parse(args[0]));
			query.addMatcherByTag("lesson");
			Vector result = query.query();
			System.out.println("result: size=" + result.size());
			for (int i = 0; i < result.size(); i++) {
				//System.out.println(((Element)result.elementAt(i)).getAttributeNode("id").getValue());
			}

			query.reset();
			query.addMatcherByTag("planning");
			// query.addMatcherByAttributeNameAndValueBeginsWith("id", "core");
			query.addMatcherByAttributeNameAndValue("priority", "high");
			query.addMatcherByAttributeNameAndValue("author", "just");
			result = query.query();
			System.out.println("result: size=" + result.size());
			for (int i = 0; i < result.size(); i++) {
				System.out.println(((Element) ((Element) result.elementAt(i)).getParentNode()).getAttributeNode("id").getValue());
			}
		} catch (Exception e) {
			System.out.println("error e=" + e);
		}
	}
}

/*
 * $Log: Query.java,v $
 * Revision 1.5  2003/01/06 00:23:49  just
 * moved devenv to linux
 *
 * Revision 1.4  2002/11/14 20:25:20  just
 * reformat of code only
 *
 * Revision 1.3  2002/01/24 23:49:58  just
 * possibly failure with ant/xbook task with inner class matcherbytag ?
 *
 * Revision 1.2  2000/08/10 19:26:58  just
 * changes for comments only
 *
 *
 */

/*
   Copyright (C)2000 Just A. van den Broecke <just@justobjects.nl>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
