/****************************************************************
 * Copyright (C)2000 - Just van den Broecke - See license below *
 ****************************************************************/
package nl.justobjects.toolkit.xml;

import org.w3c.dom.Document;

import javax.xml.transform.*;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintWriter;

/**
 * Wrapper functions to convert XML files with XSL to HTML using Xalan XSLT.
 * <p>
 * This is abouth the N-th implementation, continuously plagued by changing
 * XSL standards and XSL implementations (Xalan!). Luckily only this
 * wrapper had to change in most cases.
 * </p>
 *
 * $Id: XML2HTML.java,v 1.21 2003/11/19 15:15:15 just Exp $
 * @author $Author: just $ - Just van den Broecke - Just Objects B.V. &copy;
 */
public class XML2HTML {
	private static TransformerFactory tFactory;

	static {
		// This is the only place we are Xalan-specific...
		tFactory = new org.apache.xalan.processor.TransformerFactoryImpl();

		// We need this to resolve xsl:includes
		tFactory.setURIResolver(new EntityResolverImpl());

		tFactory.setErrorListener(new XSLErrorListener());
	}


	public static void xml2Html(Document xmlDocument, String xslUrl, PrintWriter out) throws Exception {
		// Generate the HTML using the XML and XSL URLs and the output stream
		xml2Html(xmlDocument, createTransformer(xslUrl), out);
	}

	public static void xml2Html(String xmlUrl, String xslUrl, PrintWriter out) throws Exception {
		xml2Html(xml2DOM(xmlUrl), createTransformer(xslUrl), out);
	}

	public static void xml2Html(Document xmlDocument, Document xslDocument, PrintWriter out) throws Exception {
		xml2Html(xmlDocument, createTransformer(xslDocument), out);
	}

	/** Convert an XML DOM Document to a HTMl file using an XSL DOM Document. */
	public static void xml2Html(Document xmlDocument, Transformer aTransFormer, PrintWriter out) throws Exception {
		// Open HTML file.
		StreamResult sr = openStreamResult(out);
		aTransFormer.transform(xml2InputSource(xmlDocument), sr);
		// closeStreamResult(sr);
	}

	public static void xml2Html(String xmlUrl, String xslUrl, String htmlFileName) throws Exception {
		xml2Html(xml2DOM(xmlUrl), xml2DOM(xslUrl), htmlFileName);
	}

	public static void xml2Html(Document xmlDocument, String xslUrl, String htmlFileName) throws Exception {
		xml2Html(xmlDocument, xml2DOM(xslUrl), htmlFileName);
	}

	/** Convert an XML DOM Document to a HTMl file using an XSL DOM Document. */
	public static void xml2Html(Document xmlDocument, Document xslDocument, String htmlFilePath) throws Exception {
		// Create any output dirs if not existing.
		createFilePath(htmlFilePath);

		// Open HTML file.
		StreamResult sr = openStreamResult(htmlFilePath);

		Transformer transformer = createTransformer(xslDocument);

		// Provide some extra params for the XSL stylesheet
		File file = new File(htmlFilePath);

		// Some stylesheets require the output directory params
		String outDir = ".";
		if (file.getParent() != null) {
			outDir = file.getParent();
		}
		transformer.setParameter("OUTDIR", outDir);

		// Some stylesheets require the output filename (just the name, no path) params
		transformer.setParameter("OUTFILE", file.getName());

		// Do the transform
		transformer.transform(xml2InputSource(xmlDocument), sr);

		// Close the file (causes weird stuff when not doing this!).
		closeStreamResult(sr);
	}


	/** Create directory path if none exists for a filename. */
	private static void createFilePath(String fileName) throws Exception {
		p("createFilePath: " + fileName);
		File file = new File(fileName);
		if (file.getParent() != null) {
			if (!(new File(file.getParent()).exists())) {
				new File(file.getParent()).mkdirs();
			}
		}
	}

	private static void closeStreamResult(StreamResult aStreamResult) throws Exception {
		aStreamResult.getOutputStream().close();
	}

	private static StreamResult openStreamResult(String aFileName) throws Exception {
		return new StreamResult(new FileOutputStream(aFileName));
	}

	private static StreamResult openStreamResult(PrintWriter aPrintWriter) throws Exception {
		return new StreamResult(aPrintWriter);
	}

	private static Transformer createTransformer(String anXSLFile) throws Exception {
		// Set up the XSLTProcessor to use XercesLiaison.
		p("creating transformer " + anXSLFile);
		return createTransformer(xml2DOM(anXSLFile));
	}

	private static Transformer createTransformer(Document anXSLDocument) throws Exception {
		Transformer transformer = tFactory.newTransformer(new DOMSource(anXSLDocument));
		transformer.setErrorListener(new XSLErrorListener());
		return transformer;
	}

	private static Document xml2DOM(String xmlURL) throws Exception {
		return DOMUtil.parse(xmlURL);
	}

	private static DOMSource xml2InputSource(String xmlURL) throws Exception {
		return xml2InputSource(xml2DOM(xmlURL));
	}

	private static DOMSource xml2InputSource(Document anXMLDocument) throws Exception {
		return new DOMSource(anXMLDocument);
	}

	private static void p(String s) {
		// System.out.println("XML2HTML: " + s);
	}

	static private class XSLErrorListener implements ErrorListener {

		public void warning(TransformerException ex) {
			System.err.println("[XSL Warning] loc=" +
					getLocationString(ex) + ": " +
					ex.getMessage());
		}

		/** Error. */
		public void error(TransformerException ex) {
			System.err.println("[XSL Error] loc=" +
					getLocationString(ex) + ": " +
					ex.getMessage());
		}

		/** Fatal error. */
		public void fatalError(TransformerException ex) throws TransformerException {
			System.err.println("[XSL Fatal Error] loc=" +
					getLocationString(ex) + ": " +
					ex.getMessage());
			//throw ex;
		}

		/** Returns a string of the location. */
		private String getLocationString(TransformerException ex) {
			StringBuffer str = new StringBuffer();
			SourceLocator sourceLocator = ex.getLocator();
			if (sourceLocator != null) {

				String systemId = sourceLocator.getSystemId();
				if (systemId != null) {
					int index = systemId.lastIndexOf('/');
					if (index != -1)
						systemId = systemId.substring(index + 1);
					str.append(systemId);
				}
				str.append(" line:");
				str.append(sourceLocator.getLineNumber());
				str.append(" char:");
				str.append(sourceLocator.getColumnNumber());
			} else {
				str.append(ex.getMessage() + " location " + ex.getLocationAsString());

			}

			return str.toString();

		} // getLocationString(SAXParseException):String

	}

	/** For command line util to generate HTML from XML with XSL. */
	public static void main(String args[]) {
		if (args.length != 3) {
			System.out.println("Usage: XML2HTML <xmluri> <xsluri> <html output file>");
			System.exit(-1);
		}

		try {
			xml2Html(args[0], args[1], args[2]);
		} catch (Exception e) {
			System.out.println("Exception: " + e);
			//e.printStackTrace();
		}
	}
}


/*
  * $Log: XML2HTML.java,v $
  * Revision 1.21  2003/11/19 15:15:15  just
  * many changes for using JAXP (with xerces)
  *
  * Revision 1.20  2003/01/06 00:23:49  just
  * moved devenv to linux
  *
  * Revision 1.19  2002/11/14 20:25:20  just
  * reformat of code only
  *
  * Revision 1.18  2002/07/04 13:36:08  just
  * fix for setting OUTDIR in transformation parameter
  *
  * Revision 1.17  2002/07/02 21:41:23  just
  * no message
  *
  * Revision 1.16  2002/05/23 15:06:30  just
  * no message
  *
  * Revision 1.15  2002/04/14 23:33:10  just
  * no message
  *
  * Revision 1.14  2002/02/04 23:27:50  just
  * mainly changes to find resources through CLASSPATH (no cpath:/ anymore)
  *
  * Revision 1.13  2002/01/24 01:13:03  just
  * fixes for closing StreamResult on files
  *
  * Revision 1.12  2002/01/03 15:33:16  just
  * changes required for upgrade to Xalan 2.2D14
  *
  * Revision 1.11  2001/06/07 14:22:39  just
  * upd
  *
  * Revision 1.10  2001/02/23 17:28:30  just
  * bugfix make absolute file path
  *
  * Revision 1.9  2001/02/17 14:28:16  just
  * added comments and changed interface for expandIds()
  *
  * Revision 1.8  2001/02/12 23:39:59  just
  * bugfixes for generation with HTML filenames (caused truncated output files).
  *
  * Revision 1.7  2000/11/23 12:53:01  just
  * various changes for entity resolvement
  *
  * Revision 1.6  2000/09/14 11:38:27  just
  * updates
  *
  * Revision 1.5  2000/08/10 19:26:58  just
  * changes for comments only
  *
  * Revision 1.4  2000/08/04 20:56:28  just
  * changes for Xalan
  *
  * Revision 1.3  2000/08/04 13:33:04  just
  * changed to use Xalan i.s.o. Lotus
  *
  */

/*
   Copyright (C)2000 Just A. van den Broecke <just@justobjects.nl>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/