/****************************************************************
 * Copyright (C)2000 - Just van den Broecke - See license below *
 ****************************************************************/

package nl.justobjects.xbook;

import nl.justobjects.toolkit.io.IO;
import nl.justobjects.toolkit.xml.DOMUtil;
import nl.justobjects.toolkit.xml.XML2HTML;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentType;

import java.io.File;
import java.util.HashMap;

/**
 * Generates HTML file(s) from XML file.
 * <p>
 * <b>Purpose</b><br>
 * This class is a generic XML to HTML transformer that may be used for a variety
 * of DTDs and XSL scripts. It adds extra functionality over standard XSL transformation.
 * <p>
 * <b>Implementation</b><br>
 * TBS.
 * <p>
 * <b>Examples</b><br>
 * See XBook.java generate() and the xdoc scripts.
 * <p>
 * <br>
 * $Source: /var/cvs/justobjects/xbook/src/nl/justobjects/xbook/XDoc.java,v $<p>
 *
 * @version $Revision: 1.7 $
 * @author $Author: just $ - Just van den Broecke - Just Objects &copy;
 * @since $Date: 2003/01/06 10:27:22 $
 * @see <package name.classname#method>
 */
public class XDoc {

	/** Root dir: will be found through CLASSPATH. */
	static final private String XSL_HOME = "nl/justobjects/xbook/rsc/xsl";

	/** XHTML tag identifier. */
	static final private String XHTML_TAG = "<xhtml>";

	/** Cache to parse XSL documents only once for multiple transforms. */
	static final private HashMap xslDocumentCache = new HashMap(3);

	private String sourceFilePath;
	private String destinationFilePath;
	private Document sourceDocument;
	private String xslPath;


	public XDoc() {
	}

	/** Perform the task. */
	public void transform() throws Exception {
		p("transform src=" + getSourceFilePath() + " dest=" + getDestinationFilePath());

		preTransform();
		XML2HTML.xml2Html(getSourceDocument(), getXSLDocument(), getDestinationFilePath());
		postTransform();
	}

	public String getAssetsDir() throws Exception {
		String sourceAssetsDir = new File(sourceFilePath).getParent() + File.separator + "assets";
		if (new File(sourceAssetsDir).exists()) {
			return sourceAssetsDir;
		}
		return null;
	}

	public Document getSourceDocument() throws Exception {
		if (sourceDocument == null) {
			sourceDocument = DOMUtil.parse(sourceFilePath, getDTDCheck());
		}

		return sourceDocument;
	}

	public boolean getDTDCheck() throws Exception {
		// Normally DTD-checking is on, _except_ when <xhtml> escapes are used in the text.
		return (IO.grep(getSourceFilePath(), XHTML_TAG).length > 0) ? false : true;
	}

	public String getDestinationFilePath() throws Exception {
		if (destinationFilePath == null) {
			destinationFilePath = sourceFilePath.substring(0, sourceFilePath.indexOf(".xml")) + ".html";
		}
		return destinationFilePath;
	}

	public String getDTDName() throws Exception {
		DocumentType documentType = getSourceDocument().getDoctype();
		if (documentType == null) {
			throw new Exception("source XML document " + sourceFilePath + " does not have a DTD");
		}

		String dtdName = documentType.getName();
		if (dtdName == null || dtdName.length() == 0) {
			throw new Exception("source XML document " + sourceFilePath + " does not have a DTD name");
		}

		return dtdName;
	}


	public String getSourceFilePath() {
		return sourceFilePath;
	}

	public Document getXSLDocument() throws Exception {
		if (!xslDocumentCache.containsKey(getXSLPath())) {
			xslDocumentCache.put(getXSLPath(), DOMUtil.parse(getXSLPath(), false));
		}
		// Parse the XSL document only once since it may be used multiple times
		return (Document) xslDocumentCache.get(getXSLPath());
	}

	public String getXSLPath() throws Exception {
		if (xslPath == null) {
			// If no path set, construct it from the base path and
			// the DTD name + 2html.xsl. For example doc.dtd will have
			// nl/justobjects/xbook/rsc/xsl/doc2html.xsl
			xslPath = XSL_HOME + "/" + getDTDName() + "2html.xsl";
		}

		return xslPath;
	}

	public void setSourceDocument(Document aSourceDocument) {
		sourceDocument = aSourceDocument;
	}

	public void setSourceFilePath(String aSourceFilePath) {
		sourceFilePath = aSourceFilePath;
	}

	public void setDestinationFilePath(String aDestinationFilePath) {
		destinationFilePath = aDestinationFilePath;
	}

	public void setXSLPath(String aPath) {
		xslPath = aPath;
	}

	protected void preTransform() throws Exception {
		File sourceDocumentFile = new File(getSourceFilePath());
		if (!sourceDocumentFile.exists()) {
			throw new Exception("source XML document " + getSourceFilePath() + " does not exist");
		}

		// Intialize global vars. This isn't very OO but there is in some
		// cases no other way to communicate back/forth with XSLT...
		XSLTHelper.init();

		// Set global vars in order to communicate with XSL stylesheets
		// that require exact locations of source and destination dirs.
		setDirectoryVar("srcDir", sourceDocumentFile);
		setDirectoryVar("destDir", new File(getDestinationFilePath()));
		XSLTHelper.setVar("destFile", new File(getDestinationFilePath()).getName());

		// TODO: THIS SHOULD BE MOVED TO A XSL STYLESHEET   !!!!
		String parentDir = new File(getSourceFilePath()).getParent();
		DOMUtil.expandIncludes(getSourceDocument(), parentDir, parentDir);
	}

	protected void postTransform() throws Exception {
		// Resolve all internal references for split files.
		XSLTHelper.resolveInternalLinks();
	}

	static public void p(String s) {
		System.out.println("XDoc: " + s);
	}

	private void setDirectoryVar(String aVarName, File aFile) {
		String dir = ".";
		if (aFile.getParent() != null) {
			dir = aFile.getParent();
		}
		XSLTHelper.setVar(aVarName, dir);
	}

	static public void fatal(String what, Exception e) {
		p("ERROR - " + what + " exception =" + e);
		e.printStackTrace();
		System.exit(-1);
	}

	static public void main(String args[]) {

		if (args.length == 0 || args.length > 2) {
			p("Usage: xdoc <xml file> [target html file]");
			System.exit(-1);
		}

		try {
			// Generate the album
			XDoc xDoc = new XDoc();
			xDoc.setSourceFilePath(args[0]);
			if (args.length == 2) {
				xDoc.setDestinationFilePath(args[1]);
			}
			xDoc.transform();
		}
		catch (Exception e) {
			fatal("Exception in process()", e);
		}

		System.exit(0);
	}
}

/*
 * $Log: XDoc.java,v $
 * Revision 1.7  2003/01/06 10:27:22  just
 * removed Arial fonts from default styles
 *
 * Revision 1.6  2002/07/28 11:57:49  just
 * getAssetsDir() returned wrong file name in XDoc
 *
 * Revision 1.5  2002/07/05 23:56:00  just
 * introduction of page.dtd and page2html.xsl
 *
 * Revision 1.4  2002/07/04 23:41:30  just
 * stuff for resolving internal links over split HTML
 *
 * Revision 1.3  2002/07/04 14:09:23  just
 * mainly comments
 *
 * Revision 1.2  2002/07/03 22:00:35  just
 * generic document transformer for all dtds and xsl tempaltes
 *
 * Revision 1.1  2002/07/01 22:04:00  just
 * new
 *
 *
 */

/*
   Copyright (C)2002 Just A. van den Broecke <just@justobjects.nl>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/