/****************************************************************
 * Copyright (C)2000 - Just van den Broecke - See license below *
 ****************************************************************/

package nl.justobjects.xbook;

import java.io.File;
import java.util.Enumeration;
import java.util.Properties;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import nl.justobjects.toolkit.io.IO;
import nl.justobjects.toolkit.sys.CommandLine;
import nl.justobjects.toolkit.xml.DOMUtil;
import nl.justobjects.toolkit.xml.XML2HTML;

/**
 * Generates menu-driven HTML navigation from xmenu XML file.
 * <p>
 * <b>Purpose</b><br>
 * TBS.
 * <p>
 * <b>Implementation</b><br>
 * TBS.
 * <p>
 * <b>Examples</b><br>
 * Give some code examples. Be sure to use the &lt;pre&gt; HTML tag.
 * <p>
 * <br>
 * $Source: /var/cvs/justobjects/xbook/src/nl/justobjects/xbook/XMenu.java,v $<p>
 *
 * @version $Revision: 1.4 $
 * @author $Author: just $ - Just van den Broecke - Just Objects &copy;
 * @since $Date: 2003/01/06 10:27:22 $
 * @see <package name.classname#method>
 */
 public class XMenu {
 	/** Usage string when command fails. */
	static final public String usage="Usage:\n nl.justobjects.xbook.XMenu [-v] [-lite] -i <menu XML file> -o <output dir>";
	
 	/** Global resource paths. */
	static private Properties resources = new Properties();
	
	/** Root dir: will be found through CLASSPATH. */
	static final private String resourceHome = "nl/justobjects/xbook/rsc";
	
	static private boolean verbose = false;
	
	static  {
		// XSL file to convert menu.xml to JavaScript.
		resources.put("xslPath", resourceHome + "/xsl/xmenu2javascript.xsl");
		
		// lightweight SL to generate plain HTML 
		resources.put("lite.xslPath", resourceHome + "/xsl/xmenu2html.xsl");
		
		// Output JavaScript file
		resources.put("jsFile", "xmenusetup.js");		
		
		// HTML path in resources
		resources.put("htmlPath", resourceHome + "/html");
	
		// HTML template files
		resources.put("htmlFilePaths", resourceHome + "/html/htmlfiles.properties");
		
		// Style path: use default style now.
		resources.put("stylePath", resourceHome + "/style");
		
		// File with icon file path properties.
		resources.put("iconFilePaths", resourceHome + "/style/iconfiles.properties");

		// Template CSS file (adapted for fg and bg colors)
		resources.put("cssFile", "xmenu.css");		
		
		// Default foreground (text) style
		resources.put("defaultFgColor", "white");
		
		// Default foreground (menu icons) style
		resources.put("defaultOutlineColor", "white");

		// Default background color style
		resources.put("defaultBgColor", "#000060");

		// Default poweredBy stuff
		resources.put("poweredBy", "XBook");
	}
	
	static public void generate(String[] args) throws Exception {
		CommandLine commandLine = new CommandLine(args);
		
		// Get options
		boolean lite = commandLine.getOption("lite");
		verbose = commandLine.getOption("v");
		
		// Get the two main arguments: the menu XML file and the output dir.
		String menuXML = commandLine.getStringArg("i");
		if (menuXML == null)  {
			fatal("I need a menu file (-i <menu file>)\n" + usage);
		}
		
		String destDir = commandLine.getStringArg("o");
		if (destDir == null)  {
			fatal("I need a destination dir (-o <dir>)\n" + usage);
		}
		
		dbg("parmcheck ok: menuXMLFile=" + menuXML + " outdir=" + destDir + " lite=" + lite);
		
		try  {
			// Generate the site 
			generate(menuXML, destDir, lite);
		} catch (Exception e)  {
			fatal("Exception in generate()", e);
		}
	}
	
	static public void generate(String menuXMLFile, String destinationDir) throws Exception {
		generate(menuXMLFile, destinationDir, false);
	}
	
	static public void generate(String aMenuXMLFile, String aDestinationDir, boolean lite) throws Exception {
		dbg("generate() start: menuXMLFile=" + aMenuXMLFile + " outdir=" + aDestinationDir + " lite=" + lite);
		
		File menuFile = new File(aMenuXMLFile);
		if (!menuFile.exists())  {
			throw new Exception(aMenuXMLFile + " does not exist");
		}

		// The directory of the book XML file.
		String menuFileDir = menuFile.getParent();
		dbg("menuFileDir=" + menuFileDir);
		
		Document menuDocument = DOMUtil.parse(aMenuXMLFile, true);
		Element menuElement = menuDocument.getDocumentElement();
		
		IO.mkdir(aDestinationDir);
		
		// Optionally copy the default (content frame) page (e.g. main.html)
		String mainSourceFileName = menuElement.getAttribute("default");
		String mainSourceFilePath = (menuFileDir == null) ? mainSourceFileName : menuFileDir + File.separator + mainSourceFileName;
		
		// Only copy if file present in source directory, otherwise the user is responsible
		// that the file is already present in the destination directory.
		if (new File(mainSourceFilePath).exists()) {
			String mainDestFilePath = aDestinationDir + "/" + mainSourceFileName;
			dbg("copying default page " + mainSourceFilePath + " to " + mainDestFilePath);
			IO.cp(mainSourceFilePath, mainDestFilePath);
		}

		generate(menuDocument, aDestinationDir, lite);
	}
	
	static public void generate(Document aMenuDocument, String aDestinationDir, boolean lite) throws Exception {
		if (lite)  {
			generateLite(aMenuDocument, aDestinationDir);
		} else  {
			generateDHTML(aMenuDocument, aDestinationDir);
		}
	}

	/** Generate site navigation from menu XML file. */
	static public void generateDHTML(Document menuDocument, String destinationDir) throws Exception {
		// Create destination dirs
		p("START XMenu.generateDHTML()");
		String xmenuDestinationDir = destinationDir + "/xmenu";
		IO.mkdir(xmenuDestinationDir);
		
		Element menuElement = menuDocument.getDocumentElement();
		
		// Generate the menu JavaScript file
		String jsFile = xmenuDestinationDir + "/" + resources.getProperty("jsFile");
		
		dbg("generating JavaScript to " + jsFile);
		XML2HTML.xml2Html(menuDocument, resources.getProperty("xslPath"), jsFile);

		// Get list of HTML files to be copied.
		Properties htmlFilePaths = IO.getPropertiesFromClassPath(resources.getProperty("htmlFilePaths"));

		// Copy HTML files to top directory
		dbg("copying HTML files...");
		String nextHTMLFileName = null;
		String nextHTMLFilePath = null;
		String destinationFile = null;
		String htmlPath = resources.getProperty("htmlPath");
		for (Enumeration e=htmlFilePaths.keys(); e.hasMoreElements();)  {
			nextHTMLFileName = htmlFilePaths.getProperty((String)e.nextElement());
			nextHTMLFilePath = htmlPath + "/" + nextHTMLFileName;
			destinationFile = destinationDir + "/" + nextHTMLFileName;
			// p("copy HTML resource: [" + nextHTMLFileName + "] to " + destinationFile);
			IO.resourceToFile(nextHTMLFilePath, destinationFile);
		}
		
		// Replace the <TITLE> element in index.html with xmenu-specified title
		String title = menuElement.getAttribute("title");
		// p("setting title to "+title);
		
		IO.replaceTokensInFile(destinationDir + "/index.html", "$TITLE", title);	
		
		// Get the foreground color style; if not present use default
		String fgColor = menuElement.getAttribute("fgColor");
		if (fgColor == null || fgColor.equals("") )  {
			fgColor = resources.getProperty("defaultFgColor");
		}
		
		// Get the background color style; if not present use default
		String bgColor = menuElement.getAttribute("bgColor");
		if (bgColor == null || bgColor.equals("") )  {
			bgColor = resources.getProperty("defaultBgColor");
		}
		
		// Copy image files to xmenu/style directory
		Properties iconFilePaths = IO.getPropertiesFromClassPath(resources.getProperty("iconFilePaths"));
		String styleDestinationDir = xmenuDestinationDir + "/style";
		IO.mkdir(styleDestinationDir);
		IO.mkdir(styleDestinationDir + "/win");
		IO.mkdir(styleDestinationDir + "/mac");
		dbg("copying icon files...");
		
		// Copy icon files to style directory
		String outlineColor = menuElement.getAttribute("outlineColor");
		if (outlineColor == null || outlineColor.equals("") )  {
			outlineColor = resources.getProperty("defaultOutlineColor");
		}
		String nextIconFileName = null;
		String nextIconColorFileName = null;
		String styleDestinationFileName = null;
		String styleColorPath = resources.getProperty("stylePath") + "/" + outlineColor;
		for (Enumeration e=iconFilePaths.keys(); e.hasMoreElements();)  {
			nextIconFileName = iconFilePaths.getProperty((String)e.nextElement());
			nextIconColorFileName = styleColorPath + "/" + nextIconFileName;
			styleDestinationFileName = styleDestinationDir + "/" + nextIconFileName;
			// p("copy icon resource: [" + nextIconFileName + "] to " + styleDestinationFileName);
			IO.resourceToFile(nextIconColorFileName, styleDestinationFileName);
		}

		// Get .css source and destination files
		String cssSourcePath = resources.getProperty("stylePath") + "/" + resources.getProperty("cssFile");
		String cssDestinationFile = styleDestinationDir + "/"+ resources.getProperty("cssFile");
		
		// Copies css source file to destination
		IO.resourceToFile(cssSourcePath, cssDestinationFile);
		dbg("modifying " + cssDestinationFile + " fgColor=" + fgColor + " bgColor=" + bgColor);
		IO.replaceTokensInFile(cssDestinationFile, "$BGCOLOR", bgColor);	
		IO.replaceTokensInFile(cssDestinationFile, "$FGCOLOR", fgColor);	

		// Substitute bg and fg color in .css file
		IO.replaceTokensInFile(destinationDir+"/xmenu.html", "$BGCOLOR", bgColor);	
		IO.replaceTokensInFile(destinationDir+"/xmenu.html", "$FGCOLOR", fgColor);	
		IO.replaceTokensInFile(destinationDir+"/xmenu.html", "$POWERED", resources.getProperty("poweredBy"));	
		p("END XMenu.generateDHTML()");
		
		// Generate also a lite (index-lite.html) for convenience
		p("START XMenu.generateDHTML() with index-lite.html");
		String menuFile = destinationDir + "/menu-lite.html";
		dbg("generating HTML menu to " + menuFile);
		XML2HTML.xml2Html(menuDocument, resources.getProperty("lite.xslPath"), menuFile);

		// Copy index.html file to top directory
		dbg("copying index.html file...");
		htmlFilePaths = IO.getPropertiesFromClassPath(resources.getProperty("htmlFilePaths"));
		String indexHTMLFilePath = resources.getProperty("htmlPath") + "/" + htmlFilePaths.getProperty("indexLite");
		destinationFile = destinationDir + "/index-lite.html";;
		dbg("resourceToFile: " + indexHTMLFilePath + " dest=" + destinationFile);
		IO.resourceToFile(indexHTMLFilePath, destinationFile);

		// Replace the <TITLE> element in index.html with xmenu-specified title
		title = menuElement.getAttribute("title");
		IO.replaceTokensInFile(destinationFile, "$TITLE", title);	
		
		// Replace the $MAIN var to the default attribute
		String mainSourceFileName = menuElement.getAttribute("default");
		IO.replaceTokensInFile(destinationFile, "$MAIN", mainSourceFileName);	
		
		// Replace additional tokens in menu file.
		IO.replaceTokensInFile(menuFile, "$BGCOLOR", bgColor);	
		IO.replaceTokensInFile(menuFile, "$FGCOLOR", fgColor);	
		IO.replaceTokensInFile(menuFile, "$POWERED", resources.getProperty("poweredBy"));	

		p("END XMenu.generateDHTML() with index-lite.html");

	}

	static public void generateLite(Document menuDocument, String destinationDir) throws Exception {
		p("START XMenu.generateLite()");
		IO.mkdir(destinationDir);
		
		String menuFile = destinationDir + "/menu-lite.html";
		dbg("generating HTML menu to " + menuFile);
		XML2HTML.xml2Html(menuDocument, resources.getProperty("lite.xslPath"), menuFile);

		Element menuElement = menuDocument.getDocumentElement();

		// Get the foreground color style; if not present use default
		String fgColor = menuElement.getAttribute("fgColor");
		if (fgColor == null || fgColor.equals("") )  {
			fgColor = resources.getProperty("defaultFgColor");
		}
		
		// Get the background color style; if not present use default
		String bgColor = menuElement.getAttribute("bgColor");
		if (bgColor == null || bgColor.equals("") )  {
			bgColor = resources.getProperty("defaultBgColor");
		}

		// Copy index.html file to top directory
		dbg("copying index.html file...");
		Properties htmlFilePaths = IO.getPropertiesFromClassPath(resources.getProperty("htmlFilePaths"));
		String indexHTMLFilePath = resources.getProperty("htmlPath") + "/" + htmlFilePaths.getProperty("indexLite");
		String destinationFile = destinationDir + "/index.html";;
		dbg("resourceToFile: " + indexHTMLFilePath + " dest=" + destinationFile);
		IO.resourceToFile(indexHTMLFilePath, destinationFile);
		
		// Copy help file
/*		dbg("copying xmenuhelp.html file...");
		String helpHTMLFilePath = resources.getProperty("htmlPath") + "/" + htmlFilePaths.getProperty("help");
		String destinationHelpFile = destinationDir + "/" + htmlFilePaths.getProperty("help");
		IO.resourceToFile(helpHTMLFilePath, destinationHelpFile); */

		// Replace the <TITLE> element in index.html with xmenu-specified title
		String title = menuElement.getAttribute("title");
		IO.replaceTokensInFile(destinationFile, "$TITLE", title);	
		
		// Replace the $MAIN var to the default attribute
		String mainSourceFileName = menuElement.getAttribute("default");
		IO.replaceTokensInFile(destinationFile, "$MAIN", mainSourceFileName);	

		// Get and modify .css source and destination files
/*		String xmenuDestinationDir = destinationDir + "/xmenu";
		String styleDestinationDir = xmenuDestinationDir + "/style";
		String cssSourcePath = resources.getProperty("stylePath") + "/" + resources.getProperty("cssFile");
		String cssDestinationFile = styleDestinationDir + "/"+ resources.getProperty("cssFile");
		IO.mkdir(styleDestinationDir);
		
		// Copies css source file to destination
		IO.resourceToFile(cssSourcePath, cssDestinationFile);
		dbg("modifying " + cssDestinationFile + " fgColor=" + fgColor + " bgColor=" + bgColor);
		IO.replaceTokensInFile(cssDestinationFile, "$BGCOLOR", bgColor);	
		IO.replaceTokensInFile(cssDestinationFile, "$FGCOLOR", fgColor); */	

		IO.replaceTokensInFile(menuFile, "$BGCOLOR", bgColor);	
		IO.replaceTokensInFile(menuFile, "$FGCOLOR", fgColor);	
		IO.replaceTokensInFile(menuFile, "$POWERED", resources.getProperty("poweredBy"));	
		p("END XMenu.generateLite()");
	}
	
	static public void setAdditionallyPoweredBy(String aHTMLString)  {
		resources.put("poweredBy", resources.getProperty("poweredBy") + "<br>" + aHTMLString);
	}
	
	static private void dbg(String s) {
		if (verbose)  {
			p(s);
		}
	}
	
	static private void p(String s) {
		System.out.println("XMenu: " + s);
	}
	
	static private void fatal(String what) {
		p("ERROR - " + what);
		System.exit(-1);
	}
	
	static private void fatal(String what, Exception e) {
		p("ERROR - " + what);
		if (e != null)  {
			p(" exception =" + e);
			e.printStackTrace();
		}
		System.exit(-1);
	}

	static public void main(String args[]) {

		// Pass args to generate and execute	
		try  {
			// Generate the site 
			generate(args);
		} catch (Exception e)  {
			fatal("Exception in generate()", e);
		}
	}
}

 /*
  * $Log: XMenu.java,v $
  * Revision 1.4  2003/01/06 10:27:22  just
  * removed Arial fonts from default styles
  *
  * Revision 1.3  2002/07/06 15:57:28  just
  * updates for processing a default XML document for the menu
  *
  * Revision 1.2  2002/04/28 12:48:50  just
  * xmenu integration in xbook
  *
  * Revision 1.1  2002/04/28 11:54:41  just
  * xmenu integration in xbook
  *
  * Revision 1.15  2002/03/28 14:58:15  just
  * no message
  *
  * Revision 1.14  2002/02/04 23:27:26  just
  * mainly changes to find resources through CLASSPATH (no cpath:/ anymore)
  *
  * Revision 1.13  2002/01/24 01:13:58  just
  * poweredBy stuff update
  *
  * Revision 1.12  2001/12/21 14:46:51  just
  * chg: CommandLine usage outline icon attr and xmenuctrl.html now plain HTML
  *
  * Revision 1.11  2001/12/20 00:51:26  just
  * fix for css copy
  *
  * Revision 1.10  2001/12/19 23:49:09  just
  * enhancements for  lite option (css and helpfile copy)
  *
  * Revision 1.9  2001/12/17 23:52:07  just
  * lite option and copy of default file
  *
  * Revision 1.8  2001/12/14 21:47:21  just
  * added Lite generation
  *
  * Revision 1.7  2001/02/11 20:37:33  kstroke
  * fixed copying from resources (css file)
  *
  * Revision 1.6  2001/02/11 20:32:16  kstroke
  * fixed copying from resources (html+icons)
  *
  * Revision 1.5  2001/02/11 17:10:19  kstroke
  * removed cpdir for HTML and icons: copy files to enable xmenu using from .jar
  *
  * Revision 1.4  2000/12/28 15:55:45  just
  * bugfix properties init (bgColor typo)
  *
  * Revision 1.3  2000/12/17 17:00:44  just
  * bug fixes
  *
  * Revision 1.2  2000/12/11 23:53:06  just
  * modified fgcolor/bgcolor setting
  *
  * Revision 1.1  2000/12/09 16:36:58  just
  * new
  *
  */
  
 /*
	Copyright (C)2000 Just A. van den Broecke <just@justobjects.nl>

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/