/****************************************************************
 * Copyright (C)2000 - Just van den Broecke - See license below *
 ****************************************************************/

package nl.justobjects.xbook;

import java.io.FileWriter;
import java.io.PrintWriter;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import nl.justobjects.toolkit.xml.DOMUtil;

/**
 * Encapsulates XML XMenu Document for dynamic menu-population.
 * <p>
 * <b>Purpose</b><br>
 * Allows generation of dynamic menus from a Java program or servlet/JSP.
 * <p>
 * <b>Implementation</b><br>
 * TBS.
 * <p>
 * <b>Examples</b><br>
 * Give some code examples. Be sure to use the &lt;pre&gt; HTML tag.
 * <p>
 * <br>
 * @version $Id: XMenuDocument.java,v 1.3 2003/01/06 10:27:22 just Exp $
 * @author $Author: just $ - Just van den Broecke - Just Objects &copy;
 * @since $Date: 2003/01/06 10:27:22 $
 * @see <package name.classname#method>
 */
 public class XMenuDocument {
 	/** The XML menu document */
	private Document document;
	
	/** Root: the &lt;xmenu &gt; element */
	private Element rootElement;
	
	/** Empty xmenu file for initialization */
	static final private String skeletonXMLFile = "nl/justobjects/xbook/rsc/xml/menu-skeleton.xml";

	/** Constructs empty document. */
	public XMenuDocument() throws Exception {
		this(DOMUtil.parse(skeletonXMLFile, true));
	}
	
	/** Constructs from XML xmenu file. */
	public XMenuDocument(String fileName) throws Exception {
		this(DOMUtil.parse(fileName, true));
	}

	/** Constructs from XML xmenu Document. */
	public XMenuDocument(Document theDocument) throws Exception {
		document = theDocument;
		rootElement = document.getDocumentElement();
	}
	
	/** Get the xmenu Document. */
	public Document getDocument() {
		return document;
	}
	
	/** Add document (&lt;doc/&gt; element to root menu. */ 
	public void addDocument(String label, String description, String url) throws Exception {
		addDocument(label, description, url, null);
	}
	
	/** Add document (&lt;doc/&gt; element to root menu with target frame. */ 
	public void addDocument(String label, String description, String url, String target) throws Exception {
		addDocument((Element) null, label, description, url, target);
	}
	
	/** Add document (&lt;doc/&gt; element to parent folder. */ 
	public void addDocument(Element parentFolderElement, String label, String description, String url) throws Exception {
		addDocument(parentFolderElement, label, description, url, null);
	}
	
	/** Add document (&lt;doc/&gt; element to parent folder with target frame. */ 
	public void addDocument(Element parentFolderElement, String label, String description, String url, String target) throws Exception {
		Element documentElement = document.createElement("doc");

		// Set required attributes
		documentElement.setAttribute("label", label);
		documentElement.setAttribute("url", url);
		documentElement.setAttribute("desc", description);
		
		// Set optional target frame if specified
		if (target != null)  {
			documentElement.setAttribute("target", target);
		}
		
		// Add to root or parent folder
		if (parentFolderElement == null)  {
			// Add to root/top
			rootElement.appendChild(documentElement);
		} else  {
			// Add to parent folder
			parentFolderElement.appendChild(documentElement);
		}
	}
	
	/** Add folder element to root node. */
	public Element addFolder(String label, String description) throws Exception {
		return addFolder(null, label, description);
	}
	
	/** Add subfolder element to specified folder node. */
	public Element addFolder(Element parentFolderElement, String label, String description) throws Exception {
		Element folderElement = document.createElement("folder");
		
		// Set required atributes
		folderElement.setAttribute("label", label);
		folderElement.setAttribute("desc", description);
		
		// Add to parent folder or document root element.
		if (parentFolderElement == null)  {
			rootElement.appendChild(folderElement);
		} else  {
			parentFolderElement.appendChild(folderElement);
		}
		
		// Return created folder element
		return folderElement;
	}
	
	/** Set attribute of &lt;xmenu &gt; root element. */
	public void setAttribute(String name, String value) {
		rootElement.setAttribute(name, value);
	}
	
	/** Generate navigation site from current document. */
	public void generateSite(String outputDir) throws Exception {
		generateSite(outputDir, false);
	}
	
	/** Generate navigation site from current document using lite HTML option. */
	public void generateSite(String outputDir, boolean liteOption) throws Exception {
		XMenu.generate(document, outputDir, liteOption);
	}

	/** Save Document as XML into file. */
	public void save(String outputFilePath) throws Exception {
		FileWriter fw = new FileWriter(outputFilePath);
		DOMUtil.printAsXML(document, new PrintWriter(fw));
	}
	
	/** Print util. */
	static private void p(String s) {
		System.out.println("XMenuDocument: " + s);
	}
	
	/** Error print/exit util. */
	static private void error(String what, Exception e) {
		p("ERROR - " + what + " exception =" + e);
		e.printStackTrace();
		System.exit(-1);
	}
	
	/** For testing. */
	static public void main(String args[]) {

		try  {
			// Create and populate menu document 
			XMenuDocument xMenuDocument = new XMenuDocument();
			
			xMenuDocument.setAttribute("title", "My Title");
			xMenuDocument.setAttribute("default", "main.html");
			xMenuDocument.addDocument("home", "topdoc", "main.html");
			Element f1 = xMenuDocument.addFolder("folder1", "descr");
			xMenuDocument.addFolder(f1, "f1.1", "descr");
			Element f12 = xMenuDocument.addFolder(f1, "f1.2", "descr");
			xMenuDocument.addDocument(f1, "f1d1", "f1d1desc", "f1d1.html");
			xMenuDocument.addDocument(f12, "f1d2", "f1d2desc", "f1d2.html");
			xMenuDocument.addDocument(f12, "f1d3", "f1d3desc",  "f1d3.html");
			
			Element f2 = xMenuDocument.addFolder("folder2", "descr");
			xMenuDocument.addDocument(f2, "f2d1", "f2d1desc", "f2d1.html");
			xMenuDocument.addDocument(f2, "f2d2", "f2d2desc", "f2d2.html");
			
			// Generate into complete site.
			xMenuDocument.generateSite("result");
			
			// For testing: save as XML
			xMenuDocument.save("result/menu.xml");
		} catch (Exception e)  {
			error("Exception", e);
		}
	}
}

 /*
  * $Log: XMenuDocument.java,v $
  * Revision 1.3  2003/01/06 10:27:22  just
  * removed Arial fonts from default styles
  *
  * Revision 1.2  2002/04/28 12:48:50  just
  * xmenu integration in xbook
  *
  * Revision 1.1  2002/04/28 11:54:41  just
  * xmenu integration in xbook
  *
  * Revision 1.4  2002/02/04 23:27:26  just
  * mainly changes to find resources through CLASSPATH (no cpath:/ anymore)
  *
  * Revision 1.3  2001/12/19 23:03:12  just
  * added lite option
  *
  * Revision 1.2  2001/12/14 21:47:21  just
  * added Lite generation
  *
  * Revision 1.1  2000/12/17 17:00:14  just
  * new
  *
  *
  */
  
 /*
	Copyright (C)2000 Just A. van den Broecke <just@justobjects.nl>

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/