/****************************************************************
 * Copyright (C)2000 - Just van den Broecke - See license below *
 ****************************************************************/

package nl.justobjects.xbook;

import nl.justobjects.toolkit.awt.Thumbnail;
import nl.justobjects.toolkit.io.IO;

import java.io.File;
import java.util.ArrayList;
import java.util.Properties;

/**
 * Utility class for XSLT to Java communication.
 * <p>
 * <b>Purpose</b><br>
 * This class is an external utility used by and called from XSL scripts.
 * <p>
 * <b>Implementation</b><br>
 * <p>
 * <b>Examples</b><br>
 * See the album2html.xsl script.
 * <p>
 * <br>
 * $Source: /var/cvs/justobjects/xbook/src/nl/justobjects/xbook/XSLTHelper.java,v $<p>
 *
 * @version $Revision: 1.10 $
 * @author $Author: just $ - Just van den Broecke - Just Objects &copy;
 * @since $Date: 2003/01/06 10:27:22 $
 * @see nl.justobjects.toolkit.awt.Thumbnail
 */
public class XSLTHelper {

	private static Properties vars = new Properties();
	private static Properties anchors = new Properties();
	private static ArrayList internalLinks = new ArrayList();
	private static boolean multipleOutputFiles = false;

	public static String getAnchor(String anAnchorId) {
		return anchors.getProperty(anAnchorId);
	}

	public static boolean hasAnchors() {
		return anchors.size() > 0;
	}

	public static String putAnchor(String anAnchorId, String anHTMLFile) {
		p("putAnchor: id=" + anAnchorId + " file=" + anHTMLFile);
		if (anAnchorId == null || anAnchorId.length() == 0) {
			// return a dummy unique string such that browsers will not be confused.
			return "A" + System.currentTimeMillis();
		}
		if (!anchors.containsKey(anAnchorId)) {
			p("addAnchor: id=" + anAnchorId + " file=" + anHTMLFile);
			anchors.setProperty(anAnchorId, anHTMLFile);
		}

		return anAnchorId;
	}

	public static String putInternalLinkRef(String anAnchorId, String anHTMLFilePath) {
		// Split files need to be resolved only once.
		if (!anHTMLFilePath.endsWith("-all.html") && !internalLinks.contains(anHTMLFilePath)) {
			p("putInternalLinkRef: id=" + anAnchorId + " filePath=" + anHTMLFilePath);
			internalLinks.add(anHTMLFilePath);
		}

		return "dummy";
	}

	/** Resolve internal links to anchors over split documents. */
	public static void resolveInternalLinks() {
		if (!hasMultipleOutputFiles()) {
			return;
		}

		final String INTERNAL_LINK_PATTERN = "href=\"#";
		try {
			for (int i = 0; i < internalLinks.size(); i++) {
				String nextFile = (String) internalLinks.get(i);
				p("resolveInternalLinks: file=" + nextFile);
				String[] lines = IO.FileToStringArray(nextFile);
				if (lines.length == 0) {
					p("could not convert file to string array: " + nextFile);
				}
				for (int j = 0; j < lines.length; j++) {
					String nextLine = lines[j];
					int anchorIndex = nextLine.indexOf(INTERNAL_LINK_PATTERN);

					// Test if we need to proceed with this line at all.
					if (anchorIndex != -1) {

						// As long as we find internal links proceed with the line.
						do {
							StringBuffer lineBuffer = new StringBuffer(nextLine);

							// We have an anchor pattern in the remaining string. The actual id
							// starts after the pattern.
							int anchorIdStartIndex = anchorIndex + INTERNAL_LINK_PATTERN.length();

							// Initialize our runner.
							int nextCharIndex = anchorIdStartIndex;

							// Proceed to end of anchor string.
							while (lineBuffer.charAt(nextCharIndex++) != '\"') ;

							// Extract the anchor id (without #).
							String anchorId = lineBuffer.substring(anchorIdStartIndex, nextCharIndex - 1);
							//p("resolveInternalLinks resolve anchor=" + anchorId);

							// Get the file containing the anchor from the stored anchor/file pairs.
							String anchorFile = anchors.getProperty(anchorId);
							if (anchorFile == null) {
								p("cannot resolve internal link to " + anchorId + " in file " + nextFile);
							}
							else {
								// Replace #anchor with file#anchor between the quotes if the file is different
								// from the referer file.
								if (!anchorFile.equals(nextFile)) {
									lineBuffer.replace(anchorIdStartIndex - 1, nextCharIndex - 1, anchorFile + "#" + anchorId);
								}
							}

							// Since the line length changes we need to reassign (could be more efficient...).
							nextLine = lineBuffer.toString();

							// Search for next anchor from end of last anchor.
							anchorIndex = nextLine.indexOf(INTERNAL_LINK_PATTERN, nextCharIndex-1);
						}
						while (anchorIndex != -1);

						// Replace the line
						lines[j] = nextLine;
					}
				}

				// Replace the entire file.
				IO.StringArrayToFile(nextFile, lines);
			}
		}
		catch (Exception e) {
			p("Exception while resolving internal links e=" + e);
			p("This stuff is still experimental");
			e.printStackTrace();
		}

	}

	/** Generates thumbnail image and returns thumbnail filename. */
	public static String generateThumbnail(String anInputDir, String anOutputDir, String anImageFileName) throws Exception {
		final int THUMBNAIL_SIZE = 100;
		final String THUMNAIL_FILE_EXT = "-tb.jpg";

		p("anInputDir= " + anInputDir + " anOutputDir= " + anOutputDir);
		String imageFilePath = anInputDir + File.separator + anImageFileName;

		// Give up if the image file doesn't even exist.
		if (!new File(imageFilePath).exists()) {
			throw new Exception("image file " + imageFilePath + " does not exist");
		}

		// We support only JPEG
		// TODO: check if we should also support .jpeg
		if (imageFilePath.indexOf(".jpg") == -1) {
			throw new Exception(imageFilePath + " is not a jpg file");
		}

		// Create thumbnail filename (used for URLs) and file path (used for generation)
		//String imageThumbnailFileName = imageFilePath.substring(0, imageFilePath.indexOf(".jpg")) + THUMNAIL_FILE_EXT;
		String imageThumbnailFileURL = anImageFileName.substring(0, anImageFileName.indexOf(".jpg")) + THUMNAIL_FILE_EXT;
		String imageThumbnailFilePath = anOutputDir + File.separator + imageThumbnailFileURL;

		// The output directory may not yet exist: optionally create
		IO.createFilePath(imageThumbnailFilePath);

		// Generate the thumbnail using the full paths to the filenames.
		p("creating thumbnail from " + imageFilePath + " to " + imageThumbnailFilePath);
		Thumbnail.generate(imageFilePath, imageThumbnailFilePath, THUMBNAIL_SIZE);

		// Return URL-type filename
		return imageThumbnailFileURL;
	}

	public static void init() {
		multipleOutputFiles = false;
		vars = new Properties();
		anchors = new Properties();
		internalLinks = new ArrayList();
	}

	public static boolean hasMultipleOutputFiles() {
		return multipleOutputFiles;
	}

	public static boolean enableMultipleOutputFiles() {
		multipleOutputFiles = true;
		return true;
	}

	public static String getVar(String aName) {
		//p("getVar name=" + aName + " value=" + vars.getProperty(aName));
		return vars.getProperty(aName);
	}

	public static String setVar(String aName, String aValue) {
		//p("setVar name=" + aName + " value=" + aValue);
		vars.setProperty(aName, aValue);
		return aValue;
	}

	static public void p(String s) {
		// System.out.println("XSLTHelper: " + s);
	}

}

/*
 * $Log: XSLTHelper.java,v $
 * Revision 1.10  2003/01/06 10:27:22  just
 * removed Arial fonts from default styles
 *
 * Revision 1.9  2002/08/21 21:24:48  just
 * allow sectionN element nesting upto 8 levels
 *
 * Revision 1.8  2002/07/28 11:57:49  just
 * getAssetsDir() returned wrong file name in XDoc
 *
 * Revision 1.7  2002/07/28 11:13:03  just
 * invalid file separator generated in some cases for thumbnail url
 *
 * Revision 1.6  2002/07/08 23:34:10  just
 * fix empty anchor generation (putAnchor)
 *
 * Revision 1.5  2002/07/07 15:21:23  just
 * use id attribute in sections and allow links to it
 *
 * Revision 1.4  2002/07/05 23:56:00  just
 * introduction of page.dtd and page2html.xsl
 *
 * Revision 1.3  2002/07/04 23:47:01  just
 * remove println
 *
 * Revision 1.2  2002/07/04 23:41:30  just
 * stuff for resolving internal links over split HTML
 *
 * Revision 1.1  2002/07/04 20:46:16  just
 * no message
 *
 * Revision 1.1  2002/07/04 15:34:29  just
 * new
 *
 * Revision 1.2  2002/07/04 14:02:38  just
 * mainly comments
 *
 * Revision 1.1  2002/07/03 22:00:06  just
 * external call from XsSL to generate thumbnails in album2html.xsl
 *
 */

/*
   Copyright (C)2002 Just A. van den Broecke <just@justobjects.nl>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/